local ToyPlus = LibStub("AceAddon-3.0"):NewAddon("ToyPlus", "AceConsole-3.0", "AceEvent-3.0")
local icon = LibStub("LibDBIcon-1.0")
local LibQTip = LibStub('LibQTip-1.0')

local TP_MAX_ROWS = 10
local TP_MAX_COLS = 10
local TP_MAX_TOYS = 100

local ToyPlusLDB = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("ToyPlusLDB", {
	type = "data source",
	text = "ToyPlus",
	icon = "Interface\\Icons\\INV_Misc_Toy_09",
	OnClick = function(self, button)
		if button == "LeftButton" then
			ToyPlus:Launch()
		elseif button == "RightButton" then
			InterfaceOptionsFrame_OpenToCategory("ToyPlus")
		end
	end,
	OnEnter = function(self) ToyPlus:Broker(self) end,
})

function ToyPlus:Broker(self)
	LibQTip:Release(self.tooltip)
	self.tooltip = nil
	for i = 1, #ToyPlusDB.toyName do
		local toyBtn = _G["brokerBtn"..i]
		if toyBtn then toyBtn:Hide() end
	end
	if InCombatLockdown() then return end

	ToyPlus:RefreshToys()
	local tooltip = LibQTip:Acquire("ToyPlus_Tooltip", 1, "LEFT")
	self.tooltip = tooltip
	tooltip:AddHeader('ToyPlus')
	tooltip:SetLineTextColor(1,0,0.7,1)
	tooltip:AddSeparator(1,0,0.5,1)
	tooltip:AddLine('Left-Click to toggle the toy icons bars.')
	tooltip:AddLine('Right-Click to open the configuration.')
	tooltip:AddSeparator(1,0,0.5,1)
	if not C_ToyBox.GetIsFavorite(ToyPlusDB.itemID[1]) then
		tooltip:AddLine('No favourites found. Add some via the toy list.')
	else
		tooltip:AddLine('Click a toy below to use:')
		tooltip:AddSeparator(1,0,0.5,1)
		for i = 1, #ToyPlusDB.toyName do
			if C_ToyBox.GetIsFavorite(ToyPlusDB.itemID[i]) then
				local toyName, itemID = ToyPlusDB.toyName[i], ToyPlusDB.itemID[i]
				local start, duration = GetItemCooldown(itemID)
				local tRow = i+7
				tooltip:AddLine(toyName)
				if start == 0 then tooltip:SetLineTextColor(tRow,0,0.7,1)
				else tooltip:SetLineTextColor(tRow,1,0,0) end
				tooltip:SetLineScript(tRow, "OnEnter", function(tt)
					local toyBtn = _G["brokerBtn"..i]
					if not toyBtn then
						toyBtn = CreateFrame("Button","brokerBtn"..i,self,"SecureActionButtonTemplate") 
						toyBtn:SetFrameStrata(tt:GetFrameStrata())
						toyBtn:SetFrameLevel(tt:GetFrameLevel()+1)
					end
					toyBtn:SetAllPoints(tt)
					toyBtn:Show()
					toyBtn:SetAttribute("type", "item")
					toyBtn:SetAttribute("item", toyName)
					toyBtn:SetScript("OnEnter", function()
						local tooltip = LibQTip:Acquire("ToyPlus_Tooltip", 1, "LEFT")
						if tooltip:IsShown() then
							tooltip:SetLineTextColor(tRow,1,1,0)
						end
					end)
					toyBtn:SetScript("OnLeave", function()
						local tooltip = LibQTip:Acquire("ToyPlus_Tooltip", 1, "LEFT")
						if tooltip:IsShown() then
							local start, duration = GetItemCooldown(itemID)
							if start == 0 then tooltip:SetLineTextColor(tRow,0,0.7)
							else tooltip:SetLineTextColor(tRow,1,0,0) end
						end
					end)
					toyBtn:RegisterUnitEvent("ACTIONBAR_UPDATE_COOLDOWN"); toyBtn:RegisterUnitEvent("LOSS_OF_CONTROL_UPDATE")
					toyBtn:SetScript("OnEvent", function(self, event)
						if event == "ACTIONBAR_UPDATE_COOLDOWN" or "LOSS_OF_CONTROL_UPDATE" then
							local tooltip = LibQTip:Acquire("ToyPlus_Tooltip", 1, "LEFT")
							if tooltip:IsShown() then
								local start, duration = GetItemCooldown(itemID)
								if start == 0 then tooltip:SetLineTextColor(tRow,0,0.7)
								else tooltip:SetLineTextColor(tRow,1,0,0) end
							end
						end
					end)
				end, tt)
			end
		end
	end
	tooltip:SetAutoHideDelay(1)
	tooltip:SmartAnchorTo(self)
	tooltip:Show()
end

function ToyPlus:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("ToyPlusDB", {
		profile = {
			minimap = {
				hide = false,
			},
		},
	})
	icon:Register("ToyPlus", ToyPlusLDB, self.db.profile.minimap)
	if not ToyPlus.db.profile.rows then ToyPlus.db.profile.rows = 2 end
	if not ToyPlus.db.profile.columns then ToyPlus.db.profile.columns = 5 end
	if not ToyPlus.db.profile.scale then ToyPlus.db.profile.scale = 30 end
	ToyPlus:RegisterChatCommand("toyplus", "Launch")
	ToyPlus:RegisterOptions()
end

function ToyPlus:RegisterOptions()-- Blizzard Options
	local AceConfig = LibStub("AceConfig-3.0")
	AceConfig:RegisterOptionsTable("ToyPlus", {
		type = 'group',
		args = {
			togglebutton = {
				type = 'toggle',
				order = 1,
				name = "Show Minimap Icon",
				get = function()
					return not ToyPlus.db.profile.minimap.hide
				end,
				set = ToyPlus.ToggleLDBIcon,
			},
			toyCols = {
				type = 'range',
				order = 2,
				name = "Columns",
				get = function(info)
					return ToyPlus.db.profile.columns
				end,
				set = function(info, key)
					ToyPlus.db.profile.columns = key
					ToyPlus:ToggleRows()
				end,
				min = 3, max = 10, step = 1,
			},
			toyRows = {
				type = 'range',
				order = 3,
				name = "Rows",
				get = function(info)
					return ToyPlus.db.profile.rows
				end,
				set = function(info, key)
					ToyPlus.db.profile.rows = key
					ToyPlus:ToggleRows()
				end,
				min = 1, max = 10, step = 1,
			},
			toyScale = {
				type = 'range',
				order = 4,
				name = "Scale",
				get = function(info)
					return ToyPlus.db.profile.scale
				end,
				set = function(info, key)
					ToyPlus.db.profile.scale = key
					ToyPlus:ToggleRows()
				end,
				min = 25, max = 30, step = 1,
			},

		},
	})
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ToyPlus")
end

local toyPlusFrame = CreateFrame("Frame")--Load on Startup
toyPlusFrame:RegisterEvent("ADDON_LOADED")
function ToyPlus:OnEvent(event, addonName)
	if event == "ADDON_LOADED" and addonName == "ToyPlus" then
		ToyPlusDB = ToyPlusDB or {}
		if not ToyPlusDB.toyName then ToyPlusDB.toyName, ToyPlusDB.itemID, ToyPlusDB.toyIcon = {}, {}, {}
		elseif ToyPlusDB.toyName[1] and ToyPlus.db.profile.shown == true then ToyPlus:CreateFrame(false); ToyPlus:UnregisterEvent("ADDON_LOADED") end
	end
end
toyPlusFrame:SetScript("OnEvent", ToyPlus.OnEvent)

function ToyPlus:ToggleLDBIcon()-- Toggle Minimap Icon
	ToyPlus.db.profile.minimap.hide = not ToyPlus.db.profile.minimap.hide
	if ToyPlus.db.profile.minimap.hide then
		icon:Hide("ToyPlus")
	else
		icon:Show("ToyPlus")
	end
end

function ToyPlus:ToysUpdated()
	local f = _G["ToyPlus_Popout"]
	if f then
		if C_ToyBox.GetNumLearnedDisplayedToys() > #ToyPlusDB.toyName then ToyPlus:RefreshList(0, false) end
	end
end
ToyPlus:RegisterEvent("TOYS_UPDATED", "ToysUpdated")

function ToyPlus:ToggleRows()-- Toggle Icon Rows
	local fra = _G["ToyPlusFrame"]
	if fra then
		local toyRows = ToyPlus.db.profile.rows
		local toyCols = ToyPlus.db.profile.columns
		local toyScale = ToyPlus.db.profile.scale
		local toyTotal = toyRows*toyCols
		local prevToy
		for i = 1, toyTotal do
			local toyIcon = _G["ToyPlus_Icon"..i]
			if toyIcon then
				if i == 1 then
					toyIcon:SetPoint("TOPLEFT",fra,"TOPLEFT",7,-27)
					prevToy = toyIcon
				elseif i == (toyCols+1) or i == ((toyCols*2)+1) or i == ((toyCols*3)+1) or i == ((toyCols*4)+1) or i == ((toyCols*5)+1)	 
				or i == ((toyCols*6)+1) or i == ((toyCols*7)+1) or i == ((toyCols*8)+1) or i == ((toyCols*9)+1) then 
					toyIcon:SetPoint("TOPLEFT",prevToy,"BOTTOMLEFT",0,-7)
					prevToy = toyIcon
				else
					toyIcon:SetPoint("TOPLEFT",_G["ToyPlus_Icon"..(i-1)],"TOPRIGHT",7,0) 
				end
				toyIcon:SetSize(toyScale,toyScale)
				toyIcon.texture:SetAllPoints()
			end
		end
		local startNum, endNum = 1, toyTotal
		for i = startNum, endNum do --Make icons visible
			local toyIcon = _G["ToyPlus_Icon"..i]
			if toyIcon then toyIcon:Show(); 
			toyIcon.texture:SetTexCoord(0.1,0.9,0.1,0.9)
			toyIcon.texture:SetAllPoints(toyIcon)
			toyIcon.texture:SetDrawLayer("BACKGROUND") end
		end
		startNum = endNum+1
		endNum = 100
		for i = startNum, endNum do --Hide the rest
			local toyIcon = _G["ToyPlus_Icon"..i]
			if toyIcon then toyIcon:Hide() end
		end

		local fraH = 27+(toyRows*(toyScale+7))
		fraH=floor(fraH)
		if fraH%2 ~= 0 then fraH=fraH-1 end
		fra:SetHeight(fraH)

		local fraW = 8+(toyCols*(toyScale+6.8))
		fraW=floor(fraW)
		if fraW%2 ~= 0 then fraW=fraW+1 end
		fra:SetWidth(fraW)

		local btnAddRow = _G["ToyPlusFrame_AddRow"]
		local btnLessRow = _G["ToyPlusFrame_LessRow"]
		local toyRows = ToyPlus.db.profile.rows
		if toyRows == 1 then btnLessRow:Hide(); btnAddRow:Show()
		elseif toyRows > 1 and toyRows < 10 then btnLessRow:Show(); btnAddRow:Show()
		elseif toyRows == 10 then btnLessRow:Show(); btnAddRow:Hide() end
		if toyCols > 3 then _G["ToyPlusFrame_Label"]:Show() else _G["ToyPlusFrame_Label"]:Hide() end
	end
end

function ToyPlus:RefreshToys() -- Add Toys to DB
	C_ToyBox.SetFilterCollected(true)
	C_ToyBox.ClearAllSourceTypesFiltered()--Load ToyBox and ensure proper filtering
	wipe(ToyPlusDB.toyName); wipe(ToyPlusDB.itemID); wipe(ToyPlusDB.toyIcon)
	local toyTotal = C_ToyBox.GetNumTotalDisplayedToys()
	local toyTotalLearned = C_ToyBox.GetNumLearnedDisplayedToys()
	local toyNum = 1
	if toyTotalLearned > 0 then
		for i = 1, toyTotal do
			local itemNo = C_ToyBox.GetToyFromIndex(i)
			local itemID, toyName, toyIcon, toyFave = C_ToyBox.GetToyInfo(itemNo)
			if PlayerHasToy(itemID) then
				ToyPlusDB.toyName[toyNum] = toyName
				ToyPlusDB.itemID[toyNum] = itemID
				ToyPlusDB.toyIcon[toyNum] = toyIcon
				toyNum=toyNum+1
			end
		end
	end
end

function ToyPlus:Launch()
	if C_ToyBox.GetToyFromIndex(1) ~= -1 then
		ToyPlus:RefreshToys()
		local fra = _G["ToyPlusFrame"]
		local fra2 = _G["ToyPlus_Popout"]
		if not fra then
			if InCombatLockdown() then DEFAULT_CHAT_FRAME:AddMessage("ToyPlus: Error - Can't launch while in combat.", 0, 0.7, 1); return
			else ToyPlus:CreateFrame(true) end
		else
			if fra:IsVisible() then 
				local btnPopout = _G["ToyPlusFrame_Popout"]
				fra:Hide()
				ToyPlus.db.profile.shown = false
				btnPopout.texture:SetTexture("Interface\\BUTTONS\\UI-SpellbookIcon-NextPage-Up")
				if fra2 then fra2:Hide() end 
			else
				fra:Show() 
				ToyPlus.db.profile.shown = true
			end
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("ToyPlus: Error - Toy Box hasn't been loaded or no toys found.", 0, 0.7, 1)
	end
end

function ToyPlus:ShowList()
	local fra = _G["ToyPlus_Popout"]
	local mainFra = _G["ToyPlusFrame"]
	if not fra then
		fra = CreateFrame("Frame", "ToyPlus_Popout", UIParent)
		fra:SetFrameStrata("BACKGROUND")
		fra:EnableMouse(true)
		fra:RegisterForDrag("LeftButton")
		fra:SetPoint("BOTTOMLEFT", mainFra, "BOTTOMRIGHT",-1,0)
		fra:SetSize(286,186)
		fra.texture = fra:CreateTexture()--Background
		fra.texture:SetPoint("TOPLEFT", fra, 2, -2)
		fra.texture:SetPoint("BOTTOMRIGHT", fra, -2, 2)
		fra.texture:SetTexture(0,0,0)
		fra.texture:SetAlpha(0.8)
		fra:Show()
		local scrFra = CreateFrame("ScrollFrame", "ToyPlus_ScrollFrame", fra)
		scrFra:SetPoint("TOPLEFT", 2, -2)
		scrFra:SetPoint("BOTTOMRIGHT", -2, 2)
		fra.scrollframe = scrFra
		local scrBar = CreateFrame("Slider", "ToyPlus_Slider", scrFra, "UIPanelScrollBarTemplate")
		scrBar:SetPoint("TOPLEFT", fra, "TOPRIGHT", -19, -19)
		scrBar:SetPoint("BOTTOMLEFT", fra, "BOTTOMRIGHT", 19, 19)
		scrBar:SetMinMaxValues(1, 100)
		scrBar:SetValueStep(46)
		scrBar:SetValue(0)
		scrBar:SetWidth(16)
		scrBar:SetScript("OnValueChanged", function (self, value) self:GetParent():SetVerticalScroll(value) end)
		fra.scrollbar = scrBar 
		scrFra:EnableMouseWheel(true)
		scrFra:SetScript("OnMouseWheel",
			function(self, delta) 
				local x = scrBar:GetValue(); local y = scrBar:GetValueStep()
				if delta == 1 then scrBar:SetValue(x-y)
				else scrBar:SetValue(x+y) end 
		end)
		local scrContent = CreateFrame("Frame", "ToyPlus_Content", scrFra)
		scrContent:SetSize(128,128)
		scrFra.content = scrContent 
		scrFra:SetScrollChild(scrContent)
		local backdrop = { -- Border
		  edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		  tile = false, tileSize = 8, edgeSize = 8,
		  insets = { left = 2, right = 2, top = 2, bottom = 2 } }
		fra:SetBackdrop(backdrop)
		fra:SetBackdropBorderColor(0,0.564,1,1)
		--Checkboxes
		local x, y = 1, 1
		for i = 1, #ToyPlusDB.toyName do
			local itemNo = ToyPlusDB.itemID[i]
			local itemID, toyName, toyIconStr, toyFave = C_ToyBox.GetToyInfo(itemNo)
			local chkbox = CreateFrame("CheckButton", "ToyPlus_Check"..i, scrContent, "UICheckButtonTemplate")
			local chktxt =  _G[chkbox:GetName().."Text"]
			chkbox:SetSize(25, 25)
			chktxt:SetJustifyH("Left")
			if C_ToyBox.GetIsFavorite(itemID) then chkbox:SetChecked(true) else chkbox:SetChecked(false) end
			chktxt:SetText(toyName)
			chkbox:SetPoint("TOPLEFT",scrContent,"TOPLEFT",1, y)
			chkbox:SetScript("OnClick", function(self, button)
				if button == "LeftButton" then 
					if chkbox:GetChecked(true) then ToyPlus:RefreshList(itemID, true)
					else ToyPlus:RefreshList(itemID, false) end
				end
			end)
			chkbox:SetScript("OnEnter", function(self)
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
				GameTooltip:SetToyByItemID(itemID)
			end)
			chkbox:SetScript("OnLeave", function(self)
				GameTooltip:Hide()
			end)
			y=y-18
		end
		if #ToyPlusDB.toyName > 10 then
			x = (#ToyPlusDB.toyName*18)-178
			scrBar:SetMinMaxValues(1, x)
			scrBar:SetValueStep((x/10)-1)
		else scrBar:Hide() end
	else
		if fra:IsVisible() then fra:Hide() else fra:Show() end
	end
end

function ToyPlus:RefreshList(itemID, toyFave)
	if itemID ~= 0 then C_ToyBox.SetIsFavorite(itemID, toyFave) end
	ToyPlus:RefreshToys()
	local chkID = 1
	for i = 1, #ToyPlusDB.toyName do
		local itemNo = ToyPlusDB.itemID[i]
		local itemID, toyName, toyIconStr, toyFave = C_ToyBox.GetToyInfo(itemNo)
		local chkbox = _G["ToyPlus_Check"..i]
		if not chkbox then-- Create Checkbox
			local scrContent = _G["ToyPlus_Content"]
			if not scrContent then break end
			local chkbox = CreateFrame("CheckButton", "ToyPlus_Check"..i, scrContent, "UICheckButtonTemplate")
			local chktxt =  _G[chkbox:GetName().."Text"]
			chkbox:SetSize(25, 25)
			chktxt:SetJustifyH("Left")
			if C_ToyBox.GetIsFavorite(itemID) then chkbox:SetChecked(true) else chkbox:SetChecked(false) end
			chktxt:SetText(toyName)
			local y = 19
			y=y-(18*i)
			chkbox:SetPoint("TOPLEFT",scrContent,"TOPLEFT",1, y)
			chkbox:SetScript("OnClick", function(self, button)
				if button == "LeftButton" then 
					if chkbox:GetChecked(true) then ToyPlus:RefreshList(itemID, true)
					else ToyPlus:RefreshList(itemID, false) end
				end
			end)
			chkbox:SetScript("OnEnter", function(self)
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
				GameTooltip:SetToyByItemID(itemID)
			end)
			chkbox:SetScript("OnLeave", function(self)
				GameTooltip:Hide()
			end)
		else
			local chktxt =  _G[chkbox:GetName().."Text"]
			if C_ToyBox.GetIsFavorite(itemID) then chkbox:SetChecked(true) else chkbox:SetChecked(false) end
			chktxt:SetText(toyName)
			chkbox:SetScript("OnClick", function(self, button)
				if button == "LeftButton" then 
					if chkbox:GetChecked(true) then ToyPlus:RefreshList(itemID, true)
					else ToyPlus:RefreshList(itemID, false) end
				end
			end)
			chkbox:SetScript("OnEnter", function(self)
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
				GameTooltip:SetToyByItemID(itemID)
			end)
			chkbox:SetScript("OnLeave", function(self)
				GameTooltip:Hide()
			end)
		end
		local scrBar = _G["ToyPlus_Slider"]
		if scrBar then
			if #ToyPlusDB.toyName > 10 then
				x = (#ToyPlusDB.toyName*18)-178
				scrBar:SetMinMaxValues(1, x)
				scrBar:SetValueStep((x/10)-1)
			else scrBar:Hide() end
		end
	end
	ToyPlus:RefreshIcons()
end

function ToyPlus:RefreshIcons()
	ToyPlus:RefreshToys()
	local toyRows = ToyPlus.db.profile.rows
	local toyCols = ToyPlus.db.profile.columns
	local toyTotal = toyRows*toyCols
	for i = 1, toyTotal do
		if ToyPlusDB.itemID[i] then
			local toyItemID = ToyPlusDB.itemID[i]
			local itemID, toyName, toyIconStr = C_ToyBox.GetToyInfo(toyItemID)
			local toyIcon = _G["ToyPlus_Icon"..i]
			local toyCD = _G["ToyPlus_Icon"..i.."_CD"]
			local start, duration = GetItemCooldown(itemID)
			toyCD:SetCooldown(start, duration)--Update CD
			toyIcon.texture:SetTexture(toyIconStr)
			toyIcon:SetAttribute("item", toyName)
			toyIcon:SetScript("OnEnter", function(self)
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
				GameTooltip:SetToyByItemID(itemID)
			end)
			toyIcon:SetScript("OnLeave", function(self)
				GameTooltip:Hide()
			end)
			toyIcon:SetScript("OnDragStart", function(self)
				C_ToyBox.PickupToyBoxItem(itemID)
			end)
			toyIcon:SetScript("OnEvent", function(self, event, ...)
				if event == "ACTIONBAR_UPDATE_COOLDOWN" then
					local start, duration = GetItemCooldown(itemID)
					if start ~= 0 then toyCD:SetCooldown(start, duration) end
				end
			end)
		end
	end
end

function ToyPlus:CreateFrame(checkToy)
	local fra = CreateFrame("Frame", "ToyPlusFrame", UIParent)--Toy Icons Frame
	fra:SetFrameStrata("BACKGROUND")
	fra:SetMovable(true); fra:EnableMouse(true)
	fra:RegisterForDrag("LeftButton")
	fra:SetScript("OnDragStart", fra.StartMoving)
	fra:SetScript("OnDragStop", fra.StopMovingOrSizing)
	fra:SetPoint("CENTER") 
	fra:SetSize(185,96)
	fra:SetUserPlaced(true)
	fra.texture = fra:CreateTexture()
	fra.texture:SetPoint("TOPLEFT", fra, 2, -2)
	fra.texture:SetPoint("BOTTOMRIGHT", fra, -2, 2)
	fra.texture:SetTexture(0,0,0,0.8)
	fra:SetScript("OnMouseUp", function(self, button)
		if button == "RightButton" then InterfaceOptionsFrame_OpenToCategory("ToyPlus") end
	end)
	fra:Show()
	local backdrop = {
	  edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	  tile = false, tileSize = 8, edgeSize = 8,
	  insets = { left = 2, right = 2, top = 2, bottom = 2 }
	}
	fra:SetBackdrop(backdrop)
	fra:SetBackdropBorderColor(0,0.564,1,1)
	local label = fra:CreateFontString("$parent_Label","OVERLAY","GameFontNormal")--ToyList Label
	label:SetPoint("TOPRIGHT",fra,"TOPRIGHT",-26,-7)
	label:SetText("Toy List")
	label:Hide()
	local btn = CreateFrame("BUTTON", "$parent_Close", fra, "UIPanelCloseButton")-- Close Button
	btn:SetPoint("TOPLEFT", fra, "TOPLEFT", -1, 2)
	btn:SetSize(30,30)
	btn:Show(); btn:Enable()
	btn:SetScript("OnClick", function (self, button, down) 
		if button == "LeftButton" then 
			fra:Hide()
			ToyPlus.db.profile.shown = false
			local fra2 = _G["ToyPlus_Popout"]
			if fra2 then
				fra2:Hide(); _G["ToyPlusFrame_Popout"].texture:SetTexture("Interface\\BUTTONS\\UI-SpellbookIcon-NextPage-Up")
			end
		end 
	end)
	local btnAddRow = CreateFrame("BUTTON", "$parent_AddRow", fra)-- Add Row Button
	btnAddRow:SetPoint("TOPLEFT", fra, "TOPLEFT", 32, -6)
	btnAddRow:SetAlpha(1); btnAddRow:Show(); btnAddRow:Enable()
	btnAddRow:SetSize(14,14)
	btnAddRow.texture = btnAddRow:CreateTexture()
	btnAddRow.texture:SetAllPoints(btnAddRow)
	btnAddRow.texture:SetDrawLayer("ARTWORK", -1)
	btnAddRow.texture:SetTexture("Interface\\PaperDollInfoFrame\\Character-Plus")
	btnAddRow:SetScript("OnClick", function (self, button) 
		if button == "LeftButton" then
			ToyPlus.db.profile.rows = ToyPlus.db.profile.rows + 1
			ToyPlus:ToggleRows()
		end
	end)
	local btnLessRow = CreateFrame("BUTTON", "$parent_LessRow", fra)-- Add Row Button
	btnLessRow:SetPoint("TOPLEFT", fra, "TOPLEFT", 51, -6)
	btnLessRow:SetAlpha(1); btnLessRow:Show(); btnLessRow:Enable()
	btnLessRow:SetSize(16,16)
	btnLessRow.texture = btnLessRow:CreateTexture()
	btnLessRow.texture:SetAllPoints(btnLessRow)
	btnLessRow.texture:SetDrawLayer("ARTWORK", -1)
	btnLessRow.texture:SetTexture("Interface\\WorldMap\\Dash_64")
	btnLessRow:SetScript("OnClick", function (self, button) 
		if button == "LeftButton" then
			ToyPlus.db.profile.rows = ToyPlus.db.profile.rows - 1
			ToyPlus:ToggleRows()
		end
	end)
	local btnPopout = CreateFrame("BUTTON", "$parent_Popout", fra)-- Popout Button
	btnPopout:SetPoint("TOPRIGHT", fra, "TOPRIGHT", 0, 0)
	btnPopout:SetAlpha(1); btnPopout:Show(); btnPopout:Enable()
	btnPopout:SetSize(26,26)
	btnPopout.texture = btnPopout:CreateTexture()
	btnPopout.texture:SetTexCoord(0,1,0,1)
	btnPopout.texture:SetAllPoints(btnPopout)
	btnPopout.texture:SetDrawLayer("ARTWORK", -1)
	btnPopout.texture:SetTexture("Interface\\BUTTONS\\UI-SpellbookIcon-NextPage-Up")
	btnPopout:SetScript("OnClick", function (self, button) 
		if button == "LeftButton" then
			local fra = _G["ToyPlus_Popout"]
			if not fra then	btnPopout.texture:SetTexture("Interface\\BUTTONS\\UI-SpellbookIcon-PrevPage-Up")
			elseif fra:IsVisible() then btnPopout.texture:SetTexture("Interface\\BUTTONS\\UI-SpellbookIcon-NextPage-Up")
			else btnPopout.texture:SetTexture("Interface\\BUTTONS\\UI-SpellbookIcon-PrevPage-Up") end
			ToyPlus:ShowList() 
		end
	end)
	-- Toy Buttons
	local iconH = 6
	local iconV = -27
	for i = 1, 100 do
		if ToyPlusDB.toyName[i] then
			local toyName = ToyPlusDB.toyName[i]
			local toyIconStr = ToyPlusDB.toyIcon[i]
			local itemID = ToyPlusDB.itemID[i]
			toyIcon = CreateFrame("BUTTON","ToyPlus_Icon"..i,fra, "SecureActionButtonTemplate,ActionButtonTemplate")--Icons
			toyIcon:RegisterUnitEvent("ACTIONBAR_UPDATE_COOLDOWN"); toyIcon:RegisterUnitEvent("LOSS_OF_CONTROL_UPDATE")
			toyIcon:SetAttribute("type1", "item")
			toyIcon:SetAttribute("item", toyName)
			toyIcon:RegisterForDrag("LeftButton")
			toyIcon:RegisterForClicks("LeftButtonUp", "RightButtonUp")
			toyIcon:SetSize(30,30)
			toyIcon:SetPoint("TOPLEFT",fra,"TOPLEFT",6,-27)
			toyIcon.texture = toyIcon:CreateTexture()
			toyIcon.texture:SetTexCoord(0.1,0.9,0.1,0.9)
			toyIcon.texture:SetAllPoints(toyIcon)
			toyIcon.texture:SetDrawLayer("BACKGROUND")
			toyIcon.texture:SetTexture(toyIconStr)
			local border = _G["ToyPlus_Icon"..i.."NormalTexture"]
			border:SetVertexColor(0,0.6,1,1)
			local toyCD = CreateFrame("Cooldown","$parent_CD",toyIcon, "CooldownFrameTemplate")
			toyCD:SetAllPoints()
			local start, duration = GetItemCooldown(itemID)
			if start ~= 0 then toyCD:SetCooldown(start, duration) end
			toyIcon:SetScript("OnEnter", function(self)
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
				GameTooltip:SetToyByItemID(itemID)
			end)
			toyIcon:SetScript("OnLeave", function(self)
				GameTooltip:Hide()
			end)
			toyIcon:SetScript("OnDragStart", function(self)
				C_ToyBox.PickupToyBoxItem(itemID)
			end)
			toyIcon:SetScript("OnEvent", function(self, event, ...)
				if event == "ACTIONBAR_UPDATE_COOLDOWN" or "LOSS_OF_CONTROL_UPDATE" then
					local start, duration = GetItemCooldown(itemID)
					if start ~= 0 then toyCD:SetCooldown(start, duration) end
				end
			end)
			toyIcon:SetScript("OnMouseUp", function(self, button)
				if button == "RightButton" then
					local isFavorite = C_ToyBox.GetIsFavorite(ToyPlusDB.itemID[i])
					if isFavorite then
						local menu = {
						    { text = "Remove Favorite", func = function() C_ToyBox.SetIsFavorite(ToyPlusDB.itemID[i], false); ToyPlus:RefreshList(0, false); end },
						}
						local menuFrame = CreateFrame("Frame", "ToyPlus_FaveMenu", UIParent, "UIDropDownMenuTemplate")
						EasyMenu(menu, menuFrame, "cursor", 0 , 0, "MENU");
					else
						local menu = {
						    { text = "Add Favorite", func = function() C_ToyBox.SetIsFavorite(ToyPlusDB.itemID[i], true); ToyPlus:RefreshList(0, false); end },
						}
						local menuFrame = CreateFrame("Frame", "ToyPlus_FaveMenu", UIParent, "UIDropDownMenuTemplate")
						EasyMenu(menu, menuFrame, "cursor", 0 , 0, "MENU");
					end
				end
			end)
			if checkToy then
				if PlayerHasToy(itemID) then toyIcon.texture:SetDesaturated(false) else toyIcon.texture:SetDesaturated(true) end
			end
			toyIcon:Hide()
		end
	end
	ToyPlus:ToggleRows()
	ToyPlus.db.profile.shown = true
end